unit DB97Btn;

{ TDBToolbarButton97: a data-aware TToolbarButton97
  Delphi 2 - 3, CB1
  Version 1.7 Feb-03-1999
  (C)1997-99 Christoph R. Kirchner

  Made for Toolbar97 version 1.53
  by Jordan Russell
  email:      jordanr@iname.com
  home page:  http://www.connect.net/jordanr/


Disclaimer:

  Users of this component must accept this disclaimer of warranty:
    "This unit is supplied as is. The author disclaims all warranties,
    expressed or implied, including, without limitation, the warranties
    of merchantability and of fitness for any purpose.
    The author assumes no liability for damages, direct or
    consequential, which may result from the use of this component."

  This component is donated to the public as public domain.

  This component can be freely used and distributed in commercial and
  private environments.

  The source code may be freely distributed and used. The author
  accepts no responsibility for its use or misuse.

  If you do find this component handy and you feel guilty for using
  such a great product without paying someone - sorry :)

  Please forward any comments or suggestions to Christoph Kirchner at:
  ckirchner@geocities.com

PS:

  Maybe you can find an update of this component at
  'The Delphi Component Building Site':
  http://www.geocities.com/SiliconValley/Heights/7874/delphi.htm

  Thanks to Frank Emser for:
   - A bug-report with corrected source
   - Making the Delphi 3 - version.
  Thanks to Jair Roberto Nunes da Silva for the MenuItem-property.
  Thanks to Gil Yoktan for the Language property.
}

{ to make it easier to cut&paste source for different DBxButton-types: }
{$DEFINE DBSpeedButton} 

interface

{$IFDEF VER90}
{$DEFINE D2LIKE}
{$ENDIF}
{$IFDEF VER93}
{$DEFINE D2LIKE}
{$ENDIF}

uses
  WinTypes, WinProcs, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Messages, StdCtrls, ExtCtrls, Buttons, dbconsts, DB, Menus,
  DRefresh, DBBtnCmn, TB97;


type

{ TCustomDBToolbarButton97 }

  TCustomDBToolbarButton97 = class(TToolbarButton97)
  private
    FDisableReasons: TDBBDisableReasons;
    FDataLink: TDBBtnDataLink;
    FMenuItem: TMenuItem;
    FOnMenuItemClick: TNotifyEvent;
    FConfirmDelete: Boolean;
    FButtonEnabled: Boolean;
    FDBButtonType: TDBButtonType;
    FLanguage: TDBButtonLanguage;
    FOnBeforeAction: TBeforeActionEvent;
    FOnGetEnabled: TGetEnabledEvent; { MODIFICADO DSP - 4/01/98 }
    FOnGetQuestion: TGetQuestionEvent;
    FOldOnGlyphChanged: TNotifyEvent;
    FOnEnabledChanged: TNotifyEvent;
    FRepeatTimer: TTimer;
    FCustomGlyph: Boolean;
    FInProcUpdateEnabled: Boolean;
    procedure TimerExpired(Sender: TObject);
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure SetDBButtonType(Value: TDBButtonType);
    function GetDisabled: Boolean;
    procedure SetDisabled(Value: Boolean);
    procedure SetLanguage(Value: TDBButtonLanguage);
    procedure SetMenuItem(Value: TMenuItem);
    procedure MenuItemClick(Sender: TObject);
    procedure ReadButtonEnabled(Reader: TReader);
    procedure WriteButtonEnabled(Writer: TWriter);
    function NumberOfStandardComponentName: Integer;
    function HasStandardComponentName: Boolean;
    function CalculateComponentName(NewButtonType: TDBButtonType;
      var NewName: TComponentName): Boolean;
    procedure LoadGlyph;
    function IsStoreGlyph: Boolean;
    procedure GlyphChanged(Sender: TObject);
    procedure CalcDisableReasons;
    function StandardComponentName: TComponentName;
    procedure CMEnabledChanged(var Message: TMessage);
      message CM_ENABLEDCHANGED;
  protected
    FOnAfterAction: TAfterActionEvent;
    procedure Paint; override;
    procedure Loaded; override;
    procedure DefineProperties(Filer: TFiler); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    function CtrlNamePrefix: string; virtual;
    procedure DoGetQuestion(var Question: string; var Buttons: TMsgDlgButtons;
      var HelpCtx: Longint); virtual;
    function DoAskQuestion: Boolean; virtual;

{   Calculates property Enabled: }
    procedure UpdateEnabled; virtual;

{   DoBeforeAction Calls the OnBeforeAction event: }
    procedure DoBeforeAction(var ActionIsDone: Boolean); virtual;

{   For descentants: }
    property StoreGlyph: Boolean read FCustomGlyph write FCustomGlyph;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;

{   DoAction Calls DoBeforeAction.
    Then it performs the standard-action if DataButtonType <> nbCustom.
    Same as Click, but it does not call the OnClick event: }
    procedure DoAction; virtual;
  protected { possible published: }
    property ConfirmDelete: Boolean read FConfirmDelete write FConfirmDelete;
    property DataButtonType: TDBButtonType
      read FDBButtonType write SetDBButtonType;
    property Language: TDBButtonLanguage
      read FLanguage write SetLanguage;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property MenuItem: TMenuItem read FMenuItem write SetMenuItem;
    property Disabled: Boolean read GetDisabled write SetDisabled;
    property OnGetQuestion: TGetQuestionEvent
      read FOnGetQuestion write FOnGetQuestion;
    property OnBeforeAction: TBeforeActionEvent
      read FOnBeforeAction write FOnBeforeAction;
    property OnAfterAction: TAfterActionEvent
      read FOnAfterAction write FOnAfterAction;
    property DisableReasons: TDBBDisableReasons
      read FDisableReasons write FDisableReasons;
    property OnEnabledChanged: TNotifyEvent
      read FOnEnabledChanged write FOnEnabledChanged;
    property OnGetEnabled: TGetEnabledEvent
      read FOnGetEnabled write FOnGetEnabled;
  published
    property Glyph stored IsStoreGlyph;
  end;


{ TDBToolbarButton97 }

  TDBToolbarButton97 = class(TCustomDBToolbarButton97)
  public
{   If you want to force the button to stay disabled, set the property
    Disabled to true. To set Enabled to false does only work if the
    button is enabled (sorry, but the message CM_ENABLEDCHANGED gets
    only called if Enabled changes). At designtime, you can use the
    property Enabled as usual. }
    property Disabled;
  published
    property ConfirmDelete default True;
    property DataButtonType;
    property Language default StandardDBBLanguage;
    property DataSource;
    property MenuItem;

{   Use OnGetQuestion if you want to ask the user before the action
    gets performed: }
    property OnGetQuestion;

{   Use BeforeAction instead of the Click-event if you want to be able to
    cancel the default-action by setting ActionIsDone to true.
    The Click-event is called before the DoAction-event. }
    property OnBeforeAction;

    property OnAfterAction;

{   Use DisableReasons to say on what case the button has to be disabled.
    It is set automatic if you set DataButtonType <> nbCustom.
    DisableReason  | Disable if Dataset is...
    ---------------+-------------------------
      drBOF        | EOF
      drEOF        | BOF
      drReadonly   | Readonly
      drNotEditing | Not in insert or edit-mode
      drEditing    | In insert or edit-mode
      drEmpty      | Both BOF and EOF
      drEvent      | Get the disability from the event }
    property DisableReasons;

{   Called if drEvent in DisableReasons: }
    property OnGetEnabled; 

{   Called after Enabled has changed: }
    property OnEnabledChanged;
  end;


procedure Register;

implementation

{ $R DBCTRLS} { uses DBCTRLS.RES, but that is already linked by DB.PAS }
(*
{$IFDEF WIN32}
  {$R DBSBtn.d32} { 32-bit-DBSBtn.dcr is renamed to d32 }
{$ELSE DEF WIN32}
  {$R DBSBtn.d16}
{$ENDIF DEF WIN32}
*)

{ TCustomDBToolbarButton97 }

constructor TCustomDBToolbarButton97.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csSetCaption];
  FDataLink := TDBBtnDataLink.Create;
  with FDataLink do
  begin
    OnEditingChanged := UpdateEnabled;
    OnDataSetChanged := UpdateEnabled;
    OnActiveChanged := UpdateEnabled;
  end;
  DataButtonType := nbCustom;
  FLanguage := LastSelectedDBBLanguage;
  FConfirmDelete := True;
  FButtonEnabled := True;
  FCustomGlyph := false;
  FOldOnGlyphChanged := Glyph.OnChange;
  Glyph.OnChange := GlyphChanged;
  FDisableReasons := [];
  FRepeatTimer := nil;
  FMenuItem:= nil;
  FOnGetQuestion:= nil;
  FOnBeforeAction := nil;
  FOnAfterAction := nil;
  FOnGetEnabled := nil;
  FInProcUpdateEnabled := false;
end;

destructor TCustomDBToolbarButton97.Destroy;
begin
  if (FRepeatTimer <> nil) then
    FRepeatTimer.Free;
  FDataLink.Free;
  FDataLink := nil;
  MenuItem:= nil;
  inherited Destroy;
end;

procedure TCustomDBToolbarButton97.GlyphChanged(Sender: TObject);
begin
  FCustomGlyph := true;
  if Assigned(FOldOnGlyphChanged) then FOldOnGlyphChanged(Sender);
end;

function TCustomDBToolbarButton97.IsStoreGlyph: Boolean;
begin { store only user-defined glyph: }
  result := (FDBButtonType = nbCustom) or FCustomGlyph;
end;

procedure TCustomDBToolbarButton97.LoadGlyph;
var
{$IFDEF WIN32}
{$IFNDEF D2LIKE} { Delphi >= 3.0: }
  TargetInstance: LongInt;
  NavigatorDLL: String;
{$ENDIF NDEF D2LIKE}
{$ELSE DEF WIN32}
  Buffer: array[0..79] of Char;
{$ENDIF DEF WIN32}
  ResName: string;
begin
  if (FDBButtonType = nbCustom) then
    exit;
  try
  { Load the Bitmap that DBNavigator would load: }
    FmtStr(ResName, 'dbn_%s', [BtnTypeName[FDBButtonType]]);
  {$IFDEF WIN32}
    {$IFDEF D2LIKE} { Delphi = 2.x: }
      Glyph.Handle := LoadBitmap(HInstance, PChar(ResName));
    {$ELSE DEF D2LIKE} { Delphi >= 3.0: }
(*****************************************************************************)
(* Changed by Frank Emser on June, 1st, 1997.                                *)
(* This change is due to Delphi 3.                                           *)
(* The original version will not show any Glyphs in Delphi 3.                *)
(*  This is true for design time,                                            *)
(*  this is true for runtime (when being built with runtime packages)        *)
(*  this is false only for runtime when NOT being built with runtime packages*)
(*                                                                           *)
(* Reason:                                                                   *)
(* * Delphi 3 works with packages.                                           *)
(*   These are some (very special) kind of Windows-DLLs.                     *)
(*   The component palette is created out of packages (-->DLLs)              *)
(*   This unit refers to resources of the TDBNavigator-Unit                  *)
(*   which - at design time                                                  *)
(*           and at runtime (when being built with rt-packages)-             *)
(*   is located in another package (another DLL) and therefore in these      *)
(*   cases is accessible only with another Instance handle                   *)
(*     (Exactly the one of the DLL (package)  )                              *)
(*****************************************************************************)
   // 1. Search for Run time DLL containing "dbctrls"
   // This is applicable at design time
   // OR
   // (     when the program is running stand-alone
   //   AND it is built using runtime packages.
   // )
{$IFDEF VER120} // D4:
   NavigatorDLL := 'VCLDB40.BPL';
{$ELSE DEF VER120} // D3:
   NavigatorDLL := 'VCLDB30.DPL';
{$ENDIF DEF VER120}
   TargetInstance := GetModuleHandle (PChar (NavigatorDLL));
   if TargetInstance = 0
   then begin
     // 2. Use original instance
     // This is applicable, when the program is running stand-alone
     // AND it is built without runtime packages.
     TargetInstance := HInstance;
    end;
    Glyph.Handle := LoadBitmap(TargetInstance, PChar(ResName));
    {$ENDIF DEF D2LIKE}
  {$ELSE DEF WIN32}
  { Glyph.Assign(nil); { clear }
    Glyph.Handle := LoadBitmap(HInstance, StrPCopy(Buffer, ResName));
  {$ENDIF DEF WIN32}
    NumGlyphs := 2;
    FCustomGlyph := false;
  except
  { error: do nothing }
  end;
end;

procedure TCustomDBToolbarButton97.CalcDisableReasons;
begin
  case FDBButtonType of
    nbPrior: FDisableReasons := [drBOF, drEditing, drEmpty];
    nbNext: FDisableReasons := [drEOF, drEditing, drEmpty];
    nbFirst: FDisableReasons := [drBOF, drEditing, drEmpty];
    nbLast: FDisableReasons := [drEOF, drEditing, drEmpty];
    nbInsert, nbAppend: FDisableReasons := [drReadonly, drEditing];
    nbEdit: FDisableReasons := [drReadonly, drEditing, drEmpty];
    nbCancel: FDisableReasons := [drNotEditing];
    nbPost: FDisableReasons := [drNotEditing];
    nbRefresh: FDisableReasons := [drEditing];
    nbDelete: FDisableReasons := [drReadonly, drEditing, drEmpty];
  end;
end;

function TCustomDBToolbarButton97.StandardComponentName: TComponentName;
begin
{ returns the classname without 'T' at begin, functions with decendants too: }
  result := ClassName; { 'TDBToolbarButton97' }
  if (Upcase(result[1]) = 'T') then
    System.Delete(result, 1, 1); { 'DBToolbarButton97' }
end;

function TCustomDBToolbarButton97.NumberOfStandardComponentName: Integer;
function NumberOfName(const TestName: TComponentName): Integer;
begin
  if (Length(Name) > Length(TestName)) and
     (Copy(Name, 1, Length(TestName)) = TestName) then
  begin
    try
      result := StrToInt(Copy(Name, Length(TestName) + 1, 255));
    except
      result := 0;
    end;
  end
  else
    result := 0;
end; { function NumberOfName }
begin { TCustomDBToolbarButton97.NumberOfStandardComponentName }
  result := NumberOfName(StandardComponentName);
  if (result = 0) then
    result := NumberOfName(CtrlNamePrefix + BtnName[FDBButtonType]);
end;

function HasName(const Name, TestName: TComponentName): Boolean;
begin
  if (Length(Name) > Length(TestName)) and
     (Copy(Name, 1, Length(TestName)) = TestName) then
  begin
    try
      result := (StrToInt(Copy(Name, Length(TestName) + 1, 255)) > 0);
    except
      result := false;
    end;
  end
  else
    result := (Name = TestName);
end;

function TCustomDBToolbarButton97.HasStandardComponentName: Boolean;
begin
  result :=
    HasName(Name, StandardComponentName) or
    HasName(Name, CtrlNamePrefix + BtnName[FDBButtonType]);
end;

function TCustomDBToolbarButton97.CalculateComponentName(
  NewButtonType: TDBButtonType;
  var NewName: TComponentName): Boolean;
const
  TooMuch_SomethingIsWrong = 66;
var
  Number: Integer;
begin
  result := false;
  try { ... to find new name for the component: }
    if HasStandardComponentName then
    begin
      if (NewButtonType = nbCustom) then
        NewName := StandardComponentName + '1'
      else
        NewName := CtrlNamePrefix + BtnName[NewButtonType];
      if (Owner <> nil) and (Owner.FindComponent(NewName) <> nil) then
      begin
        Number := NumberOfStandardComponentName;
        if (Number = 0) then
          Number := 1;
        repeat
          if (NewButtonType = nbCustom) then
            NewName := StandardComponentName + IntToStr(Number)
          else
            NewName := CtrlNamePrefix + BtnName[NewButtonType] +
                       IntToStr(Number);
          Inc(Number);
        until (Owner.FindComponent(NewName) = nil) or
              (Number = TooMuch_SomethingIsWrong);
        { Number = TooMuch_SomethingIsWrong should never happen, but I
          don't want to risk an endless loop in design-mode. }
        result := (Number <> TooMuch_SomethingIsWrong) and (NewName <> Name);
      end
      else
        result := (NewName <> Name);
    end;
  except
  { don't change name if error occured, no warning }
  end;
end;

procedure TCustomDBToolbarButton97.SetDBButtonType(
  Value: TDBButtonType);
var
  NewName: TComponentName;
begin
  if (Value = FDBButtonType) then
    exit;
  if (csDesigning in ComponentState) and
     CalculateComponentName(Value, NewName) then
    Name := NewName;
  if (csReading in ComponentState) or (csLoading in ComponentState) then
  begin
    FDBButtonType := Value;
    CalcDisableReasons;
    exit;
  end;
  Enabled := True;
{ Spacing := -1; }
  if (Value = nbCustom) then
    FCustomGlyph := true
  else
  begin
  {$IFDEF DBSpeedButton}
    if (Hint = '') or IsStandardBtnCaption(Hint, FLanguage) then
    { Change Hint if it was created by us automatically: }
      Hint := BtnCaption[FLanguage, Value];
  {$ELSE DEF DBSpeedButton}
    if (Caption = '') or IsStandardBtnCaption(Caption, FLanguage) then
    { Change caption if it was created by us automatically: }
      Caption := BtnCaption[FLanguage, Value];
  {$ENDIF DEF DBSpeedButton}
  end;
  Enabled := False;
  Enabled := True;
  FDBButtonType := Value;
  LoadGlyph;
  CalcDisableReasons;
end;

procedure TCustomDBToolbarButton97.SetLanguage(Value: TDBButtonLanguage);
begin
  if (Value = FLanguage) then
    exit;
  if (FDBButtonType <> nbCustom) then
  begin
  {$IFDEF DBSpeedButton}
    if (Hint = '') or IsStandardBtnCaption(Hint, FLanguage) then
    { Change Hint if it was created by us automatically: }
      Hint := BtnCaption[Value, FDBButtonType];
  {$ELSE DEF DBSpeedButton}
    if (Caption = '') or IsStandardBtnCaption(Caption, FLanguage) then
    { Change caption if it was created by us automatically: }
      Caption := BtnCaption[Value, FDBButtonType];
  {$ENDIF DEF DBSpeedButton}
  end;
  FLanguage := Value;
  LastSelectedDBBLanguage := Value;
end;

procedure TCustomDBToolbarButton97.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
     (AComponent = DataSource) then
    DataSource := nil;
  if (Operation = opRemove) and (AComponent = FMenuItem) then
    MenuItem := nil;
end;

procedure TCustomDBToolbarButton97.DoBeforeAction(var ActionIsDone: Boolean);
begin
  if (not (csDesigning in ComponentState)) and
     Assigned(FOnBeforeAction) then
    FOnBeforeAction(self, ActionIsDone);
end;

procedure TCustomDBToolbarButton97.DoGetQuestion(
  var Question: string; var Buttons: TMsgDlgButtons; var HelpCtx: Longint);
begin
  if (FDBButtonType = nbDelete) and FConfirmDelete then
  {$IFDEF WIN32}
    {$IFDEF D2LIKE} { Delphi = 2.x: }
      Question := LoadStr(SDeleteRecordQuestion)
    {$ELSE DEF D2LIKE} { Delphi >= 3.0: }
      Question := SDeleteRecordQuestion
    {$ENDIF DEF D2LIKE}
  {$ELSE DEF WIN32} { Delphi = 1.x: }
    Question := LoadStr(SDeleteRecordQuestion)
  {$ENDIF DEF WIN32}
  else
    Question := '';
  Buttons := mbOKCancel;
  HelpCtx := 0;
  if Assigned(FOnGetQuestion) then
    FOnGetQuestion(self, Question, Buttons, HelpCtx);
end;

function TCustomDBToolbarButton97.DoAskQuestion: Boolean;
var
  Question: string;
  QuestionButtons: TMsgDlgButtons;
  QuestionHelpCtx: Longint;
  QuestionResult: Longint;
begin
  DoGetQuestion(Question, QuestionButtons, QuestionHelpCtx);
  if (Question <> '') then
  begin
    QuestionResult :=
      MessageDlg(Question, mtConfirmation, QuestionButtons, QuestionHelpCtx);
    result := (QuestionResult = idOk) or (QuestionResult = idYes);
  end
  else
    result := true;
end;

procedure TCustomDBToolbarButton97.DoAction;
var
  DataSet: TDataSet;
  Cancel: Boolean;
  Error: Boolean;
begin
  if not DoAskQuestion then
    exit;
  Cancel := (FDBButtonType = nbCustom);
  try
    DoBeforeAction(Cancel);
    if (not Cancel) and (DataSource <> nil) and
       (DataSource.State <> dsInactive) then
    begin
      DataSet := DataSource.DataSet;
      with DataSet do
      begin
        case FDBButtonType of
          nbPrior: Prior;
          nbNext: Next;
          nbFirst: First;
          nbLast: Last;
          nbInsert: Insert;
          nbAppend: Append;
          nbEdit: Edit;
          nbCancel: Cancel;
          nbPost: Post;
          nbRefresh: RefreshDataset(DataSet);
          nbDelete: Delete;
        end;
      end;
    end;
    Error := false;
  except
    Error := true;
    if Assigned(FOnAfterAction) then
      FOnAfterAction(self, Error);
    if Error then
      raise;
    Error := true;
  end;
  if not Error and not Cancel and Assigned(FOnAfterAction) then
    FOnAfterAction(self, Error);
end;

procedure TCustomDBToolbarButton97.Click;
begin
  inherited Click;
  DoAction;
end;

procedure TCustomDBToolbarButton97.UpdateEnabled;
var
  PossibleDisableReasons: TDBBDisableReasons;
  GetEnable: Boolean;
  WasEnabled: Boolean;
begin
  if (csDesigning in ComponentState) then
    exit;
  if (csDestroying in ComponentState) then
    exit;
  if not FButtonEnabled then
    exit;
  FInProcUpdateEnabled := true;
  try
   WasEnabled := Enabled;
   if FDataLink.Active then
   begin
     PossibleDisableReasons := [];
     if FDataLink.DataSet.BOF then
       Include(PossibleDisableReasons, drBOF);
     if FDataLink.DataSet.EOF then
       Include(PossibleDisableReasons, drEOF);
     if not FDataLink.DataSet.CanModify then
       Include(PossibleDisableReasons, drReadonly);
     if FDataLink.DataSet.BOF and FDataLink.DataSet.EOF then
       Include(PossibleDisableReasons, drEmpty);
     if FDataLink.Editing then
       Include(PossibleDisableReasons, drEditing)
     else
       Include(PossibleDisableReasons, drNotEditing);
   { MODIFICADO DSP - 4/01/98 }
     GetEnable := ((FDisableReasons - [drEvent])* PossibleDisableReasons = []);
     if (drEvent in FDisableReasons) and (Assigned(FOnGetEnabled)) then begin
       FOnGetEnabled(Self, GetEnable);
     end;
     Enabled := GetEnable;
   end
   else
     Enabled := false;
   FButtonEnabled := true;
   if FMenuItem <> nil then
     FMenuItem.Enabled:= Enabled;
   if (WasEnabled <> Enabled) and Assigned(FOnEnabledChanged) then
     FOnEnabledChanged(self);
  finally
    FInProcUpdateEnabled := false;
  end;
end;

procedure TCustomDBToolbarButton97.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if (not FInProcUpdateEnabled) and
     (not (csLoading in ComponentState)) and
     (not (csDestroying in ComponentState)) then
  begin
    FButtonEnabled := Enabled;
    UpdateEnabled;
  end;
end;

function TCustomDBToolbarButton97.GetDisabled: Boolean;
begin
  result := not FButtonEnabled;
end;

procedure TCustomDBToolbarButton97.SetDisabled(Value: Boolean);
begin
  if (Value = FButtonEnabled) then
  begin
    FButtonEnabled := not Value;
    Enabled := not Value;
  end;
end;

procedure TCustomDBToolbarButton97.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if not (csLoading in ComponentState) then
    UpdateEnabled;
{$IFDEF WIN32}
  if Value <> nil then Value.FreeNotification(Self);
{$ENDIF DEF WIN32}
end;

function TCustomDBToolbarButton97.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TCustomDBToolbarButton97.ReadButtonEnabled(Reader: TReader);
begin
  FButtonEnabled := Reader.ReadBoolean;
end;

procedure TCustomDBToolbarButton97.WriteButtonEnabled(Writer: TWriter);
begin
  Writer.WriteBoolean(FButtonEnabled);
end;

procedure TCustomDBToolbarButton97.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty('RuntimeEnabled', ReadButtonEnabled, WriteButtonEnabled,
                       true);
end;

procedure TCustomDBToolbarButton97.Loaded;
begin
  inherited Loaded;
  if Glyph.Empty then { no user-defined glyph: }
    LoadGlyph; { load standard glyph }
  Enabled := FButtonEnabled; {}
  UpdateEnabled;
end;

procedure TCustomDBToolbarButton97.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbLeft) then
  begin
    if (FRepeatTimer = nil) and BtnNeedsTimer[FDBButtonType] then
    begin
      FRepeatTimer := TTimer.Create(Self);
      FRepeatTimer.OnTimer := TimerExpired;
      FRepeatTimer.Interval := TimerInitRepeatPause;
      FRepeatTimer.Enabled := true;
    end;
  end;
end;

procedure TCustomDBToolbarButton97.MouseUp(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  if (FRepeatTimer <> nil) then
  begin
    FRepeatTimer.Free;
    FRepeatTimer := nil;
  end;
end;

procedure TCustomDBToolbarButton97.Paint;
begin
  if (FRepeatTimer <> nil) then
//  FRepeatTimer.Enabled := Down; Down is NOT like TSpeedButton.Down
    FRepeatTimer.Enabled := (FState = bsDown);
  inherited Paint;
end;

procedure TCustomDBToolbarButton97.TimerExpired(Sender: TObject);
begin
  FRepeatTimer.Interval := TimerRepeatPause;
  try
//  Click; Click is NOT like TSpeedButton.Click:
    DoAction; // instead, call of OnClick is needed too
  except
    FRepeatTimer.Free;
    FRepeatTimer := nil;
    raise;
  end;
end;

procedure TCustomDBToolbarButton97.SetMenuItem(Value: TMenuItem);
begin
  if (Value <> FMenuItem) then
  begin
    if not (csDesigning in ComponentState) then
    begin
      if Assigned(FMenuItem) and
         not (csDestroying in FMenuItem.ComponentState) then
        FMenuItem.OnClick := FOnMenuItemClick;
      if Assigned(Value) then
      begin
        FOnMenuItemClick := Value.OnClick;
        Value.OnClick := MenuItemClick;
      end
      else
        FOnMenuItemClick := nil;
    end;
    FMenuItem := Value;
  end;
end;

procedure TCustomDBToolbarButton97.MenuItemClick(Sender: TObject);
begin
  Click;
  if Assigned(FOnMenuItemClick) then
    FOnMenuItemClick(Sender);
end;

function TCustomDBToolbarButton97.CtrlNamePrefix: string;
begin
  result := 'dbtb97Btn';
end;


{ Register ------------------------------------------------------------------ }


procedure Register;
begin
  RegisterComponents('Toolbar97', [TDBToolbarButton97]);
end;

end.
