unit DBBtnCmn;

{ Used by TDBBitBtn and TDBSpeedButton
  Delphi 1 - 4, CB1
  Version 1.7 Feb-03-1999
  (C)1997-99 Christoph R. Kirchner
}

{ Choose language for default button captions: }

{$DEFINE USEnglish}
{ $DEFINE Dutch }
{ $DEFINE Hebrew }
{ $DEFINE Portugues}
{ $DEFINE German}
{ $DEFINE Spanish}


interface

uses
  Dialogs,
  DB;


type

  TDBButtonType = (
    nbCustom,
    nbFirst, nbPrior, nbNext, nbLast,
    nbInsert, nbAppend, nbDelete,
    nbEdit,
    nbPost, nbCancel,
    nbRefresh);

  TDBButtonLanguage = (
    blUSEnglish,
    blDutch,
    blHebrew,
    blPortugues,
    blGerman,
    blSpanish);

{ MODIFICADO DSP - 4/01/98 - 1.6x (+drEvent) }
  TDBBDisableReason = (
    drBOF, drEOF, drReadonly,
    drNotEditing, drEditing, drEmpty, drEvent);
  TDBBDisableReasons = set of TDBBDisableReason;

  TGetQuestionEvent =
    procedure (Sender: TObject;
               var Question: string;
               var Buttons: TMsgDlgButtons;
               var HelpCtx: Longint) of object;

  TBeforeActionEvent =
    procedure (Sender: TObject; var ActionIsDone: Boolean) of object;

  TAfterActionEvent =
    procedure (Sender: TObject; var Error: Boolean) of object;

{ MODIFICADO DSP - 4/01/98 - 1.6x (new) }
  TGetEnabledEvent =
    procedure (Sender: TObject; var Enabled: Boolean) of object;


const

{ timer: }

  TimerInitRepeatPause = 400;  { pause before repeat timer (ms) }
  TimerRepeatPause     = 100;  { interval (ms)}

{ Caption / Hint: }

  BtnCaption: array[TDBButtonLanguage, TDBButtonType] of string[15] =
  (
  { USEnglish: }
    ('', 'First', 'Prior', 'Next', 'Last',
     'New', 'New', 'Delete', 'Edit',
     'Save', 'Cancel', 'Refresh'),
  { Dutch: }
    ('', 'Eerste', 'Vorige', 'Volgende', 'Laatste',
     'Nieuw', 'Nieuw', 'Verwijder', 'Bewerken',
     'Opslaan', 'Annuleren', 'Vernieuw'),
  { Hebrew: }
    ('', '', '', '', '',
     '', '', '', '',
     '', '', ''),
  { Portugues: }
    ('', 'Primeiro', 'Anterior', 'Prximo', 'ltimo',
     'Insere', 'Insere', 'Exclui', 'Edita',
     'Salva', 'Cancela', 'Atualiza'),
  { German: }
    ('', 'Anfang', 'Zurck', 'Vor', 'Ende',
     'Neu', 'Neu', 'Lschen', 'ndern',
     'Sichern', 'Abbrechen', 'Aktualisieren'),
  { Spanish: }
    ('', 'Primero', 'Anterior', 'Siguiente', 'ltimo',
     'Insertar', 'Insertar', 'Borrar', 'Editar',
     'Guardar', 'Cancelar', 'Actualizar')
   );

{$IFDEF USEnglish}
  StandardDBBLanguage = blUSEnglish;
{$ENDIF DEF USEnglish}
{$IFDEF Dutch}
  StandardDBBLanguage = blDutch;
{$ENDIF DEF Dutch}
{$IFDEF Hebrew}
  StandardDBBLanguage = blHebrew;
{$ENDIF DEF Hebrew}
{$IFDEF Portugues}
  StandardDBBLanguage = blPortugues;
{$ENDIF DEF Portugues}
{$IFDEF German}
  StandardDBBLanguage = blGerman;
{$ENDIF DEF German}
{$IFDEF Spanish}
  StandardDBBLanguage = blSpanish;
{$ENDIF DEF Spanish}

  LastSelectedDBBLanguage: TDBButtonLanguage = StandardDBBLanguage;

{ Component.Name: }

{ MODIFICADO DSP - 4/01/98 - 1.6x (+Spanish) }
  {$IFDEF Spanish}
  BtnName: array[TDBButtonType] of string =
    ('', 'Primero', 'Anterior', 'Siguiente', 'Ultimo',
     'Insertar', 'Insertar', 'Borrar', 'Editar',
     'Guardar', 'Cancelar', 'Refrescar');
  {$ELSE}
  BtnName: array[TDBButtonType] of string =
    ('', 'First', 'Prior', 'Next', 'Last',
     'New', 'Append', 'Delete', 'Edit',
     'Save', 'Cancel', 'Refresh');
  {$ENDIF}

{ for internal use: }

  BtnTypeName: array[TDBButtonType] of PChar =
    ('', 'FIRST', 'PRIOR', 'NEXT', 'LAST',
     'INSERT', 'INSERT', 'DELETE', 'EDIT',
     'POST', 'CANCEL', 'REFRESH');

  BtnNeedsTimer: array[TDBButtonType] of Boolean =
    (false, false, true, true, false,
     false, false, false, false,
     false, false, false);

type

{ TDBBtnDataLink }

  TEvent =
    procedure of object;

  TDBBtnDataLink = class(TDataLink)
  private
    FOnEditingChanged: TEvent;
    FOnDataSetChanged: TEvent;
    FOnActiveChanged: TEvent;
  protected
    procedure EditingChanged; override;
    procedure DataSetChanged; override;
    procedure ActiveChanged; override;
  public
    constructor Create;
    property OnEditingChanged: TEvent
      read FOnEditingChanged write FOnEditingChanged;
    property OnDataSetChanged: TEvent
      read FOnDataSetChanged write FOnDataSetChanged;
    property OnActiveChanged: TEvent
      read FOnActiveChanged write FOnActiveChanged;
  end;


function IsStandardBtnCaption(s: string; Language: TDBButtonLanguage): Boolean;

implementation


function IsStandardBtnCaption(s: string; Language: TDBButtonLanguage): Boolean;
var
  DBButtonType: TDBButtonType;
begin
  for DBButtonType := Low(TDBButtonType) to High(TDBButtonType) do
    if (BtnCaption[Language, DBButtonType] = s) then
    begin
      result := true;
      exit;
    end;
  result := false;
end;


{ TDBBtnDataLink }

constructor TDBBtnDataLink.Create;
begin
  inherited Create;
  FOnEditingChanged := nil;
  FOnDataSetChanged := nil;
  FOnActiveChanged := nil;
end;

procedure TDBBtnDataLink.EditingChanged;
begin
  if Assigned(FOnEditingChanged) then FOnEditingChanged;
end;

procedure TDBBtnDataLink.DataSetChanged;
begin
  if Assigned(FOnDataSetChanged) then FOnDataSetChanged;
end;

procedure TDBBtnDataLink.ActiveChanged;
begin
  if Assigned(FOnActiveChanged) then FOnActiveChanged;
end;

end.
