unit DRefresh;

{ Version 1.07  Mar-04-1998
  (C) 1997-99 Christoph R. Kirchner

  Made to refresh a dataset that got modified by another program
  or by another TDataset-component in the same application.
  If DbiForceReread() fails, the dataset gets refreshed by closing
  and opening the cursor again.

  This unit is donated to the public as public domain.
  This unit can be freely used and distributed in commercial and
  private environments.
  The author accepts no responsibility for its use or misuse.

  Please forward any comments or suggestions to Christoph Kirchner at:
  ckirchner@geocities.com

  Maybe you can find an update at:
  'The Delphi Component Building Site'
  http://www.geocities.com/SiliconValley/Heights/7874/delphi.htm
}


interface

uses
{$IFDEF WIN32}
  Windows, BDE,
{$ELSE DEF WIN32}
  WinTypes, WinProcs, DBITypes, DBIProcs,
{$ENDIF DEF WIN32}
  SysUtils, Classes, Controls, DB, DBTables;

function RefreshDataset(Dataset: TDataset): Boolean;

implementation

{Detarmining compatibility}
{$IFDEF VER90}
{$DEFINE D2LIKE}
{$ENDIF}
{$IFDEF VER93}
{$DEFINE D2LIKE}
{$ENDIF}


type

{$IFDEF WIN32}
{$IFDEF D2LIKE} { Delphi 2.0x: }
  IllegalTypeCastedDataSet = class(TComponent)
  public
    FFields: TList;
    FDataSources: TList;
    FFieldDefs: TFieldDefs;
    FBuffers: PBufferList;
    FBufListSize: Integer;
    FBufferCount: Integer;
    FRecordCount: Integer;
    FActiveRecord: Integer;
    FCurrentRecord: Integer;
  end;
{$ELSE DEF D2LIKE} { Delphi >= 3.0: }
  {$IFDEF VER120} // D4:
    IllegalTypeCastedDataSet = class(TComponent)
    public
      FFields: Pointer;
      FAggFields: Pointer;
      FFieldDefs: Pointer;
      FFieldDefList: Pointer;
      FFieldList: Pointer;
      FDataSources: Pointer;
      FFirstDataLink: Pointer;
      FBufferCount: Integer;
      FRecordCount: Integer;
      FActiveRecord: Integer;
      FCurrentRecord: Integer;
    end;
  {$ELSE DEF VER120} // D3:
    IllegalTypeCastedDataSet = class(TComponent)
    public
      FFields: Pointer;
      FFieldDefs: Pointer;
      FDataSources: Pointer;
      FFirstDataLink: Pointer;
      FBufferCount: Integer;
      FRecordCount: Integer;
      FActiveRecord: Integer;
      FCurrentRecord: Integer;
    end;
  {$ENDIF DEF VER120}
{$ENDIF DEF D2LIKE}
{$ELSE DEF WIN32} { Delphi 1.x: }
  IllegalTypeCastedDataSet = class(TComponent)
  public
    FFields: TList;
    FDataSources: TList;
    FFieldDefs: TFieldDefs;
    FBuffers: PBufferList;
    FBufListSize: Integer;
    FBufferCount: Integer;
    FRecordCount: Integer;
    FActiveRecord: Integer;
    FCurrentRecord: Integer;
  end;
{$ENDIF DEF WIN32}

function GetActiveRecord(ADataSet: TDataSet): Integer;
begin
  with IllegalTypeCastedDataSet(ADataSet) do
    Result := FActiveRecord;
end;

procedure SetActiveRecord(ADataSet: TDataSet; Value: Integer);
begin
  with IllegalTypeCastedDataSet(ADataSet) do
    FActiveRecord := Value;
end;

(* legal, but incredible slow and with side-effects:
function GetActiveRecord(ADataSet: TDataSet): Integer;
begin
  with TDataLink.Create do
  try
    DataSource := TDataSource.Create(ADataSet);
    try
      DataSource.DataSet := ADataSet;
      BufferCount := 1000; { to set FFirstRecord to 0 - not a good idea }
      Result := ActiveRecord;
    finally
      DataSource.Free;
      DataSource := nil;
    end;
  finally
    Free;
  end;
end;

procedure SetActiveRecord(ADataSet: TDataSet; Value: Integer);
begin
  with TDataLink.Create do
  begin
    DataSource := TDataSource.Create(ADataSet);
    DataSource.DataSet := ADataSet;
    ActiveRecord := Value;
    DataSource.Free;
    DataSource := nil;
    Free;
  end;
end;
*)
(*
type

{$IFDEF WIN32}
{$IFDEF D2LIKE} { Delphi 2.0x: }
  TPublicDataSet = class(TBDEDataSet)
{$ELSE DEF D2LIKE} { Delphi >= 3.0: }
  TPublicDataSet = class(TDataSet)
{$ENDIF DEF D2LIKE}
{$ELSE DEF WIN32} { Delphi 1.x: }
  TPublicDataSet = class(TDataSet)
{$ENDIF DEF WIN32}
  public
    procedure DoCloseCursor;
    procedure DoOpenCursor;
  end;

procedure TPublicDataSet.DoCloseCursor;
begin
  CloseCursor;
end;

procedure TPublicDataSet.DoOpenCursor;
begin
  OpenCursor;
end;
*)

function RefreshDataset(Dataset: TDataset): Boolean;
var
  PriorBookmark: TBookmark;
  CurrentBookmark: TBookmark;
  NextBookmark: TBookmark;
  OldActiveRecord: Integer;
  i: Integer;
  CouldGotoBookmark: Boolean;
  ForceCloseAndOpen: Boolean;
function UniDirectional: Boolean;
begin
  result := (Dataset is TQuery) and TQuery(Dataset).UniDirectional;
end;
begin
  result := false;
  if (Dataset = nil) then
    exit;
{$IFDEF WIN32}
{$IFDEF D2LIKE} { Delphi 2.0x: }
  with Dataset do
  begin
{$ELSE DEF D2LIKE} { Delphi >= 3.0: }
  if not (Dataset is TBDEDataSet) then
  begin
    Dataset.Refresh;
    exit;
  end;
  with TBDEDataSet(Dataset) do
  begin
{$ENDIF DEF D2LIKE}
{$ELSE DEF WIN32} { Delphi 1.x: }
  with Dataset do
  begin
{$ENDIF DEF WIN32}
    if (State <> dsBrowse) then
      exit; { please try later }
    result := true;
    ForceCloseAndOpen := (Dataset is TQuery);
    DBICheckRefresh;
    DisableControls;
    try
      UpdateCursorPos;
    { Try to refresh all buffers for the dataset with DbiForceReread.
      If the dataset is a TQuery, the result set must be living,
      the tables must be local (Paradox or dBASE) and have a unique index. }
      if not ForceCloseAndOpen and (DbiForceReread(Handle) = 0) then
        Resync([])
      else
      begin
      { DbiForceReread did not work (SQL-Based or not alive).
        Re-read the data by closing and opening the DataSet: }
        CurrentBookmark := nil;
        PriorBookmark := nil;
        NextBookmark := nil;
        try
        { Store position: }
          OldActiveRecord := GetActiveRecord(Dataset); { save pos in grids }
          CurrentBookmark := GetBookmark; { save current position }
          if not BOF and not UniDirectional then
          begin
            Prior;
            PriorBookmark := GetBookmark; { save another position }
            Next;
          end;
          if not EOF then
          begin
            Next;
            NextBookmark := GetBookmark;
            Prior;
          end;
        { Re-open Dataset: }
        {$IFDEF WIN32}
        (** )
          if TDBDataSet(DataSet).SQLBased then
          begin
          { clear buffers and close Handle: }
            TPublicDataSet(DataSet).CloseCursor;
          { Execute SQL again: }
            TPublicDataSet(DataSet).OpenCursor;
          end
          else
          begin
        (**)
          Close;
          Open;
         { end; {}
        {$ELSE DEF WIN32}
          Close;
          Open;
        {$ENDIF DEF WIN32}
        { Restore position: }
          try
            CouldGotoBookmark :=
              (CurrentBookmark <> nil) and
              (DbiSetToBookmark(Handle, CurrentBookmark) = 0);
          except
            CouldGotoBookmark := false; { no raise }
          end;
          if CouldGotoBookmark then
            Resync([]);
          if not CouldGotoBookmark and (PriorBookmark <> nil) then
          begin
            try
              CouldGotoBookmark :=
                (DbiSetToBookmark(Handle, PriorBookmark) = 0);
            except
              CouldGotoBookmark := false; { no raise }
            end;
            if CouldGotoBookmark then
            begin
              Resync([]);
              if not EOF then
                Next;
            end;
          end;
          if not CouldGotoBookmark and (NextBookmark <> nil) then
          begin
            try
              CouldGotoBookmark :=
                (DbiSetToBookmark(Handle, NextBookmark) = 0);
            except
              CouldGotoBookmark := false; { no raise }
            end;
            if CouldGotoBookmark then
            begin
              Resync([]);
              if not BOF and not UniDirectional then
                Prior;
            end;
          end;
        finally
          if (CurrentBookmark <> nil) then FreeBookmark(CurrentBookmark);
          if (PriorBookmark <> nil) then FreeBookmark(PriorBookmark);
          if (NextBookmark <> nil) then FreeBookmark(NextBookmark);
        end;
      { Adjust position for TDBGrid: }
        if CouldGotoBookmark then
        begin
          for i := 1 to OldActiveRecord do
            if BOF then
            begin
              OldActiveRecord := i;
              break;
            end
            else
              Prior;
          for i := 1 to OldActiveRecord do
            if EOF then
              break
            else
              Next;
        end;
      end;
    finally
      EnableControls;
    end;
  end;
end;

end.
