unit RInsBBtn;

{ TDBRepInsBitBtn: a TDBBitBtn to repeat insert
  Delphi 1 - 3, CB1
  Version 1.7 Feb-03-1999
  (C)1997-99 Christoph R. Kirchner

Disclaimer:

  Users of this component must accept this disclaimer of warranty:
    "This unit is supplied as is. The author disclaims all warranties,
    expressed or implied, including, without limitation, the warranties
    of merchantability and of fitness for any purpose.
    The author assumes no liability for damages, direct or
    consequential, which may result from the use of this component."

  This component is donated to the public as public domain.

  This component can be freely used and distributed in commercial and
  private environments.

  The source code may be freely distributed and used. The author
  accepts no responsibility for its use or misuse.

  If you do find this component handy and you feel guilty for using
  such a great product without paying someone - sorry :)

  Please forward any comments or suggestions to Christoph Kirchner at:
  ckirchner@geocities.com

PS:

  Maybe you can find an update of this component at
  'The Delphi Component Building Site':
  http://www.geocities.com/SiliconValley/Heights/7874/delphi.htm
}

interface

{$IFDEF VER90}
{$DEFINE D2LIKE}
{$ENDIF}
{$IFDEF VER93}
{$DEFINE D2LIKE}
{$ENDIF}

uses
  WinTypes, WinProcs, SysUtils, Classes, Graphics, Controls,
  Dialogs, Messages, StdCtrls, ExtCtrls, Buttons, dbconsts, DB,
  Menus, DBBtnCmn, DBBBtn, RIBtnRes
{$IFDEF WIN32}
  {$IFDEF D2LIKE} { Delphi = 2.x: }
    , Libconst;
  {$ELSE DEF D2LIKE} { Delphi >= 3.0: }
    ;
  {$ENDIF DEF D2LIKE}
{$ELSE DEF WIN32} { Delphi = 1.x: }
  , Libconst;
{$ENDIF DEF WIN32}


type

  TDBRepInsBitBtn = class(TCustomDBBitBtn)
  private
    FFirstInput: TWinControl;
    FLastInput: TWinControl;
    FOldLastInputOnExit: TNotifyEvent;
    FRepeatInsert: Boolean;
    procedure LastInputOnExit(Sender: TObject);
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure UpdateEnabled; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure DoAction; override;
  published
    property FirstInput: TWinControl read FFirstInput write FFirstInput;
    property LastInput: TWinControl read FLastInput write FLastInput;
    property Language default StandardDBBLanguage;
    property DataSource;
    property MenuItem;
    property OnGetQuestion;
    property OnBeforeAction;
    property OnAfterAction;
    property OnEnabledChanged;
    property OnGetEnabled;
  end;


procedure Register;


implementation

{$IFDEF WIN32}
  {$R RInsBBtn.D32} { 32-bit-RInsBBtn.DCR is renamed to D32 }
{$ELSE DEF WIN32}
  {$R RInsBBtn.D16}
{$ENDIF DEF WIN32}

{$IFDEF WIN32}
  {$IFNDEF D2LIKE} { Delphi >= 3.0: }

uses ComCtrls;

const
{ RegisterPanel = 'Datensteuerung'; { german }
  RegisterPanel = 'Data Controls';

  {$ENDIF NDEF D2LIKE}
{$ENDIF DEF WIN32}

type

  TWinControlX = class(TWinControl)
  public
    property OnExit;
  end;

constructor TDBRepInsBitBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  DataButtonType := nbAppend;
  try
    Glyph.Handle := LoadBitmap(HInstance, 'REPINSBUTTON');
    StoreGlyph := false;
    NumGlyphs := 2;
  except
    { error: do nothing }
  end;
  FRepeatInsert := false;
end;

procedure TDBRepInsBitBtn.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FFirstInput) then
    FirstInput := nil;
  if (Operation = opRemove) and (AComponent = FLastInput) then
  begin
    FLastInput := nil;
    FOldLastInputOnExit := nil;
  end;
end;

procedure TDBRepInsBitBtn.DoAction;
var
  Cancel: Boolean;
  Error: Boolean;
begin
  if not DoAskQuestion then
    exit;
  try
    Cancel := false;
    DoBeforeAction(Cancel);
    if (not Cancel) and (DataSource <> nil) and
       (DataSource.State <> dsInactive) then
    begin
      if (FFirstInput <> nil) then begin
        { If the component is in a PageControl, change the page
          DSP 5/1/97 }
        if FFirstInput.Parent is TTabSheet then
          with TTabSheet(FFirstInput.Parent) do
            PageControl.ActivePage := TTabSheet(FFirstInput.Parent);
        FFirstInput.SetFocus;
      end;
      DataSource.DataSet.Append;
      if (FLastInput <> nil) then
      begin
        FRepeatInsert := true;
        FOldLastInputOnExit := TWinControlX(FLastInput).OnExit;
        TWinControlX(FLastInput).OnExit := LastInputOnExit;
      end;
    end;
    Error := false;
  except
    if Assigned(FOnAfterAction) then
    begin
      Error := true;
      FOnAfterAction(self, Error);
    end;
    if Error then
      raise;
    Error := true;
  end;
  if not Error and Assigned(FOnAfterAction) then
    FOnAfterAction(self, Error);
end;

procedure TDBRepInsBitBtn.LastInputOnExit(Sender: TObject);
begin
  if Assigned(FOldLastInputOnExit) then
    FOldLastInputOnExit(Sender);
  with DataSource do
    if (State = dsInsert) then
    begin
      DataSet.Post;
      DoAction;
    end;
end;

procedure TDBRepInsBitBtn.UpdateEnabled;
begin
  inherited UpdateEnabled;
  if FRepeatInsert and (FLastInput <> nil) then
  begin
    FRepeatInsert := false;
    TWinControlX(FLastInput).OnExit := FOldLastInputOnExit;
    FOldLastInputOnExit := nil;
  end;
end;


{ Register ------------------------------------------------------------------ }


procedure Register;
begin
{$IFDEF WIN32}
  {$IFDEF D2LIKE} { Delphi = 2.x: }
    RegisterComponents(LoadStr(srDControls), [TDBRepInsBitBtn]);
  {$ELSE DEF D2LIKE} { Delphi >= 3.0: }
    RegisterComponents(RegisterPanel, [TDBRepInsBitBtn]);
  {$ENDIF DEF D2LIKE}
{$ELSE DEF WIN32} { Delphi = 1.x: }
  RegisterComponents(LoadStr(srDControls), [TDBRepInsBitBtn]);
{$ENDIF DEF WIN32}
end;

end.
